/*
A Rectangle class implementation
11/15/20
*/

public class Rectangle {

	private int width;
	private int height;

	public Rectangle() {
		//Call (defer to) two-args constructor
		this(0, 0);
	}

	public Rectangle(int aWidth, int aHeight) {
		//Constructing a Rectangle: aWidth by aHeight
		this.width = aWidth;
		this.height = aHeight;
	}

	//------------------------------------------
	//Getters

	public int getWidth() {
		return width;
	}

	public int getHeight() {
		return height;
	}

	//------------------------------------------
	//Setters

	public void setWidth(int aWidth) {
		width = aWidth;
	}

	public void setHeight(int aHeight) {
		height = aHeight;
	}

	//------------------------------------------
	//Common

	public String toString() {
		//Return a "nice" display string
		return "" + width + " x " + height;
	}

	//------------------------------------------
	//Methods Using Conditional Statements

	public boolean isSquare() {
		return width == height;
	}

	public void doubleSmallestDimension() {
		/*Note: if dims are equal, okay
		to double either*/
		if (isWider())
			height = 2 * height;
		else
			width = 2 * width;
	}

	public boolean isWiderTry1() {
		//Return true if we are wider (w >= h)
		//width and height are our ivars
		if (width > height)
			return true;
		else
			return false;
	}

	public boolean isWider() {
		/* Improved. Because (width > height)
		produces a boolean -- so return it
		without the if/else (not needed) */
		return width > height;
	}

	public boolean hasInitialDimensions() {
		/*&& is a logical operator for AND.
		It means the left expression
		AND the right expression must be true */
		return (width == 0) && (height == 0);
	}

	public void print(boolean option) {
		//if arg is true, print details, else basic
		if (option)
			printDetails();
		else
			printDimensions();
	}

	public boolean canContain(Rectangle otherRec) {
		/* 	Return true if this rec can contain "otherRec".
			Assume we can turn (rotate) this rec.
			So we can contain it if:
				if our max (dimension) is > otherRec's max
				AND if our min is > otherRec's min
			We'll make use of helper methods
		*/
		int otherMax, otherMin;
		otherMax = otherRec.getMax();
		otherMin = otherRec.getMin();
		return this.isMaxLargerThan(otherMax)
				&& this.isMinLargerThan(otherMin);
	}

	public boolean canFitInside(Rectangle otherRec) {
		/* 	Can we fit inside the param "otherRec"
			We can reuse code!
			Note that if the otherRec "canContain" this Rec,
			then yes we canFitInside
		*/
		return otherRec.canContain(this);
	}

	//------------------------------------------
	//Helpers For Methods Using Conditional Statements

	public boolean isMaxLargerThan(int length) {
		//Return true if our max dimension is larger than the param
		return this.getMax() > length;
	}

	public boolean isMinLargerThan(int length) {
		//Return true if our min dimension is larger than the param
		return this.getMin() > length;
	}

	public boolean canContainMin(int length) {
		//Return true if our max dimension is larger than the param
		return this.width > length;
	}

	public int getMax() {
		//return max dimension
		if (width > height)
			return width;
		else
			return height;
	}

	public int getMin() {
		//return min dimension
		if (width < height)
			return width;
		else
			return height;
	}

	public void printDimensions() {
		prn("Width: " + width);
		prn("Height: " + height);
	}

	public void printDetails() {

		//First print the basics
		printDimensions();

		//Now print a rich mix of info about this rec
		if (width == 0 && height == 0)
			prn("Dims are initial (condition 1)");

		if (hasInitialDimensions())
			prn("Dims are initial (condition 2)");

		if (width > 10)
			prn("Width is greater than 10");

		if ((width + height) > 20)
			prn("Dim sum is greater than 20");

		if (isWider())
			prn("Greater width than height");
		else
			prn("Not greater width than height");
	}

	//------------------------------------------
	//General Helpers

	public static void prn(Object o) {
		System.out.println(o);
	}

}